#
#  Copyright 1996-2011 Jim Pirzyk / pirzyk.org
#  All rights reserved.
#
#  Security::group - group file control module
#
package Security::group;

use Security::passwd;
use Security::util::print;
use Security::util::read_config;
use Security::util::users;
use Exporter;
use strict;
use warnings "all";

our @ISA = qw(Exporter);
our @EXPORT_OK = qw( init check print );

our (%badmembers, %badlogin, %badgid, %gid0, %nopass, %dupgid, %defaultpw, %dupgpasswd, %oldpass, $group0);

sub init {
	print ( STDERR  "In Security::group::init()\n") if ( $main::opts{'d'} );

	&read_config('ignoregpasswd');
	&read_config('ignoregid');
	&read_config('ignoredupgid');
	&read_config('ignoredupgpasswd');

	# If we have yet to read this in, then do it now (might be from Security::passwd::init()
	&read_config('defaultpass')
		if ( ! exists $main::config{'defaultpass'} || ! scalar @{$main::config{'defaultpass'}} );

	# If we have not set group0 in the OS specific configs, set a default here.
	$main::group0 = 'root' if ( ! defined $main::group0 );

	$main::login_length = 31
		if ( ! defined ($main::login_length) );
}

sub check {
	print ( STDERR "In Security::group::check()\n") if ( $main::opts{'d'} );

	my ($oldname, $oldgid);
	$oldgid = -5;
	foreach my $name ( Security::util::groups::get('name') ) {
		my $passwd = Security::util::groups::get('name', $name, 'passwd');
		my $gid = Security::util::groups::get('name', $name, 'gid');

		# Check to see who has GIDs of 0.
		$gid0{$name} = 1 
			if (! $gid && $name ne $main::group0 &&
				!grep (/^\Q${name}\E$/, @{$main::config{'ignoregid'}}));

		# Check to see which group accounts are without passwords. 
		$nopass{$name} = '(group)' if ($passwd eq '' &&
				!grep (/^\Q${name}\E$/, @{$main::config{'ignoregpasswd'}}));

		# Check to see which group entries have duplicate passwords.  
		if ( $passwd ne '' && $passwd !~ /$Security::passwd::passwd_disable_regex/ ) {
			if ( exists $oldpass{$passwd} ) {
				$dupgpasswd{$name} = 'and ' . $oldpass{$passwd}
					if (!grep (/^\Q${name}\E$/, @{$main::config{'ignoredupgpasswd'}}));
				$oldpass{$passwd} .= ' ' . $name
			} else {
				$oldpass{$passwd} = $name;
			}
		}

		# Check to see who has group entries with default passwords.  The
		# admin will define the list of default encrypted passwords in
		# the config file.
		$defaultpw{$name} = 1 if (ref($main::config{'defaultpass'}) eq 'ARRAY' &&
				grep (/^\Q${passwd}\E$/, @{$main::config{'defaultpass'}}));

		# Check to see what accounts have duplicate GIDs (except the 
		# root account, which was handled earlier.  
		$dupgid{$name} = $oldname if ( $oldgid eq $gid && $gid &&
				!grep (/^\Q${name}\E$/, @{$main::config{'ignoredupgid'}}));

		# Flag any accounts that have invalid field values.  This idea comes
		# from the OpenBSD 5.0 security script.
		$badlogin{$name} = 1
			if ( length $name > $main::login_length || $name !~ /$Security::passwd::login_regex/ ); 

		# Flag GIDs that are negative, too large or otherwise not numeric
		# This idea comes from the OpenBSD 5.0 security script.
		$badgid{$name} = 1
			if ( $gid < 0 || $gid > $Security::passwd::id_max || $gid !~ /$Security::passwd::id_regex/ );

		# Now check the members of the group to see if they exist
		my $mbrs = Security::util::groups::get('name', $name, 'members');
		foreach my $m ( @{$mbrs} ) {
			if ( ! Security::util::users::get('name', $m, 'name') ) {
				if ( exists $badmembers{$name} ) {
					$badmembers{$name} = " and $m";
				} else {
					$badmembers{$name} = $m;
				}
			}
		}

		$oldgid = $gid;
		$oldname = $name;
	}
}

sub print () {
	print ( STDERR  "In Security::group::print()\n") if ( $main::opts{'d'} );

	&print_data ("\n\nchecking for invalid group names:\n", %badlogin);
	&print_data ("\n\nchecking for invalid GIDs:\n", %badgid);
	&print_data ("\n\nchecking for GIDs of 0:\n", %gid0);
	&print_data ("\n\nchecking for accounts with no passwords:\n", %nopass);
	&print_data ("\n\nchecking for accounts with default passwords:\n", %defaultpw);
	&print_data ("\n\nchecking for duplicate GIDs:\n", %dupgid);
	&print_data ("\n\nchecking for duplicate passwords:\n", %dupgpasswd);
	&print_data ("\n\nchecking for invalid group members:\n", %badmembers);
}

1;

__END__

=head1 NAME

Security::group - group file control module

=head1 SYNOPSIS

Security::group::init()

Security::group::check()

Security::group::print()

=head1 DESCRIPTION

Looks for group entries that have no password set.

Looks for group entries that have the same password, as comparing the hash
values.

Looks for group entries that have a known password, as compared to a list of
configured hashes.

Looks for group entries with duplicate GIDs.

Looks for group entries with bad login names.

Looks for group entries with bad GIDs.

Looks for group entries with members that do not exist in passwd file.

=head1 API

This module implements the default API:

=over 4

=item C<init()>

Calls C<read_config (ignoregpasswd)> to get the list of group entries
to ignore that have no password.

Calls C<read_config (ignoregid)> to get the list of group entries to
ignore that share GID 0.

Calls C<read_config (ignoredupgid)> to get the list of group entries to
ignore that share GIDs other than 0.

Calls C<read_config (ignoredupgpasswd)> to get the list of group entries
to ignore that share passwords.

Calls C<read_config (defaultpass)> to get the list of password hashes
that we know of.

=item C<check()>

Traverses the group entries via C<Security::util::groups::get()> looking
for no passwords, shared GIDs and passwords, known passwords, bad GIDs
and bad login names.

=item C<print()>

Calls C<print_data ()> to print out the list of group entries other than
root (or wheel) that have GID 0.

Calls C<print_data ()> to print out the list of group entries that do not
have a password set.

Calls C<print_data ()> to print out the list of group entries that have a
known password.

Calls C<print_data ()> to print out the list of group entries that have
duplicate GIDs.

Calls C<print_data ()> to print out the list of group entries that have
duplicate passwords.

Calls C<print_data ()> to print out the list of group entries that have
bad names.

Calls C<print_data ()> to print out the list of group entries that have
bad GIDs.

=back 

=head1 CONFIGURATION OPTIONS

The script will call C<read_config> on the following config file options:

=over 4

=item C<ignoregpasswd>

List of group entries to ignore if they have no password.

=item C<ignoregid>

List of group entries to ignore that share GID 0.

=item C<ignoredupgid>

List of group entries to ignore that share NON 0 GIDs.

=item C<ignoredupgpasswd>

List of group entries to ignore that share a password.

=item C<defaultpass>

List of password hashes to use as the list of known passwords.
Shared with the C<Security::passwd> module.

=back

=head1 HISTORY

The idea of checking the group file like the passwd file was
suggested in the OpenBSD 5.0 security script.

=head1 SEE ALSO

L<Security::util::groups>

=cut

