#
#  Copyright 1996-2011 Jim Pirzyk / pirzyk.org
#  All rights reserved.
#
#  Security::pkgchk - Solaris pkgchk control module
#
package Security::pkgchk;

use Security::util::print;
use Security::util::read_config;
use Sys::Filesystem;
use Exporter;
use strict;
use warnings "all";

our @ISA = qw(Exporter);
our @EXPORT_OK = qw( init check print );

our (%perms, @idir, $pkgchk);

sub init {
	print ( STDERR "In Security::pkgchk::init()\n") if ( $main::opts{'d'} );

	# Configurations for checking files against pkgchk command
	&read_config('ignorepkgfiles');
	$pkgchk = '/usr/sbin/pkgchk -n -q' if ( -f '/usr/sbin/pkgchk' );

	# Ignore any errors for filesystems that are pcfs
	my $fs = new Sys::Filesystem;
	my @filesystems = $fs->filesystems();
	for (@filesystems) {
		if ( $fs->format($_) eq 'pcfs' ) {
			push @idir, $_;
			print STDERR "\tPruning pkgchk at $_\n" 
				if ( $main::opts{'d'} );
		}
	}
}

sub _check_dir {
	my ($dir) = @_;

	map { return 0 if ( $dir =~ /^$_/ ); } @idir;
	return 1;
}

sub check {
	my ($dir, $FP);
	print ( STDERR "In Security::pkgchk::check()\n") if ( $main::opts{'d'} );

	return if ( ! defined $pkgchk );

	print ( STDERR "\tRunning cmd $pkgchk\n") if ( $main::opts{'d'} );
	if (open ($FP, $pkgchk . ' 2>&1 |')) {
		while (<$FP>) {
			chop;
			if ( $_ =~ /ERROR: /o ) {
				$dir = $_;
				$dir =~ s/^ERROR: //go;
				$dir = undef
					if ( grep (/^\Q${dir}\E$/, @{$main::config{'ignorepkgfiles'}}) );
			} elsif ( defined $dir && length ($dir) > 1 ) {
				if ( m/^\s+(permissions|owner name|group name|modtime) <(.*)> expected <(.*)> actual$/ ) {
					$perms{$dir}->{$1} = "($2:$3)"
						if ( &_check_dir($dir) );
				} elsif ( m/^\s+file (cksum|size|type) <(.*)> expected <(.*)> actual$/ ) {
					$perms{$dir}->{'file ' . $1} = "($2:$3)"
						if ( &_check_dir($dir) );
				} elsif ( m/^\s+pathname / ) {
					s/^\s+pathname //;
					$perms{$dir}->{'pathname'} = $_
						if ( &_check_dir($dir) );
				}
			} elsif ( defined $dir ) {
				print ( STDERR "\tDo not know what to do with $_\n\tdir = $dir\n") if ( $main::opts{'d'} );
			}
		}
		close $FP;
	} else {
		print ( STDERR "ERROR: Running cmd $pkgchk -n -q\n") if ( $main::opts{'d'} );
	}
}

sub print {
	print ( STDERR  "In Security::pkgchk::print()\n") if ( $main::opts{'d'} );

	&print_data ("\n\nchecking pkgchk command output:\n", %perms);
}

1;

__END__

=head1 NAME

Security::pkgchk - Solaris pkgchk control module

=head1 SYNOPSIS

Security::pkgchk::init();

Security::pkgchk::check();

Security::pkgchk::print();

=head1 DESCRIPTION

This module will take the output from pkgchk and standardizes the output.

=head1 API

This module implements the default API:

=over 4

=item C<init()>

Looks for the location of the C<pkgchk(1)> command.

Calls L<Sys::Filesystem::filesystems> to find any filesystems of type
C<pcfs> that are currently mounted.  Will ignore any ownership, group
or mode errors for files under this directory.

=item C<check()>

Runs the C<pkgchk(1)> command with C<-q> arguments and processes the output.
It will report any errors found with the files installed on the system.

=item C<print()>

Calls C<print_data ()> to print out the mismatches from the C<pkgchk(1)>
output.

=back

=head1 CONFIGURATION OPTIONS

The script will call C<read_config> on the following config file options:

=over 4

=item C<ignorepkgfiles>

List of files to ignore errors returned from C<pkgchk>

=back

=head1 SEE ALSO

L<pkgchk(1)>

L<Sys::Filesystem>

=cut

